<?php

namespace IMATHUZH\Qfq\Core\Report;

use IMATHUZH\Qfq\Core\Helper\KeyValueStringParser;
use IMATHUZH\Qfq\Core\Store\Store;


/**
 * Class ColumnScript
 * @package IMATHUZH\Qfq\Core\Report
 *
 * Executes a function of an external script. Column parameters:
 *  scr: path to script
 *  f: function name
 *
 * All parameters are passed to the function as the first argument.
 * The second argument passed is an instance of ScriptFunctions which acts as an interface to QFQ functionality.
 */
class ColumnScript {
    public function render(string $columnValue) {

        // Parse arguments
        $param = KeyValueStringParser::parse($columnValue, PARAM_TOKEN_DELIMITER, PARAM_DELIMITER);
        if (empty($param[TOKEN_SCRIPT])) {
            throw new \UserReportException("Missing script path.", ERROR_MISSING_VALUE);
        }
        if (empty($param[TOKEN_FUNCTION_CALL])) {
            throw new \UserReportException("Missing function name.", ERROR_MISSING_VALUE);
        }

        // include script
        $scriptPath = getcwd() . '/' . $param[TOKEN_SCRIPT];
        if (!is_readable($scriptPath)) {
            throw new \UserReportException("Can't read script file.", ERROR_IO_READ_FILE);
        }
        try {
            if((include_once $scriptPath) === false)
            {
                throw new \Exception('Include failed.');
            }
        } catch (\Exception | \Error $e) {
            throw new \UserReportException(json_encode([
                ERROR_MESSAGE_TO_USER => 'Error during reading script file.',
                ERROR_MESSAGE_TO_DEVELOPER => "Error meassage:\n" . $e->getMessage()]));
        }

        // execute function, write output to buffer
        if (!function_exists($param[TOKEN_FUNCTION_CALL])) {
            throw new \UserReportException("Function doesn't exist.", ERROR_IO_READ_FILE);
        }
        ob_start();
        try {
            $return = call_user_func_array($param[TOKEN_FUNCTION_CALL], [$param, new ScriptFunctions()]);
            $output = ob_get_clean();
        } catch (\Exception | \Error $e) {
            ob_end_clean();
            throw new \UserReportException(json_encode([
                ERROR_MESSAGE_TO_USER => 'Function execution failed.',
                ERROR_MESSAGE_TO_DEVELOPER => "Error message:\n" . $e->getMessage() . "\n\nFunction: " . $param[TOKEN_FUNCTION_CALL] . "\n\nParameters:\n" . print_r($param,true)]));
        }

        // return buffer output and fill store
        Store::setStore($return, STORE_CLIENT, true);
        return $output;
    }
}

/**
 * Class ScriptFunctions
 * @package IMATHUZH\Qfq\Core\Report
 *
 * An instance of this class is passed as the last argument to every external function call.
 * WARNING: Be aware that this acts as a "public" interface to QFQ. All changes made in here might break existing applications.
 */
class ScriptFunctions {
    public static function apiCall(string $method, string $url, string $data = '', array $header = [], int $timeout = 5) {
        return RestClient::callApiCurl($method, $url, $data, $header, $timeout);
    }
    public static function getVar($key, $useStores = STORE_USE_DEFAULT, $sanitizeClass = '', &$foundInStore = '',
                                  $typeMessageViolate = SANITIZE_TYPE_MESSAGE_VIOLATE_CLASS) {
        return Store::getInstance()->getVar($key, $useStores, $sanitizeClass, $foundInStore,
                                  $typeMessageViolate);
    }
}
